--
-- Elektronika dla Wszystkich
-- Szkoa Konstruktorw, zadanie 244 (EdW 06/2016)
-- ESP8266 IoT: WiFi Power Socket / Gniazdko sterowane przez WiFi
-- Copyright (c) 2016 by Marcin 'Malcom' Malich
-- Released under the MIT license
--

function Button(pin, active)

	local self = {
		Pin = pin,
		Active = active
	}

	local _pressed = false

	local _debounceDelay = 50
	local _debounceTimer = 5
	local _onDownCallback
	local _onUpCallback


	local _init = function()
		print("Button: init: " .. pin)
		gpio.mode(pin, gpio.INT, gpio.PULLUP)
		gpio.trig(pin, "down", _onTriggerDown)
		-- read initial status
		_pressed = gpio.read(pin) == active
	end

	local _OnTrigger = function(cur, value, callback, next, trigger)
		print("Button: OnTrigger: " .. pin .. ", " .. cur)

		gpio.trig(pin, "none")
		tmr.alarm(_debounceTimer, _debounceDelay, tmr.ALARM_SINGLE,
		function()
			if gpio.read(pin) == value then
				gpio.trig(pin, next, trigger)
			else
				trigger()
			end
		end)

		callback()
	end

	-- triggers function can't be local

	_onTriggerDown = function()
		_OnTrigger("down", active, function(t)
			_pressed = true
			if _onDownCallback ~= nil then
				_onDownCallback()
			end
		end,
		"up", _onTriggerUp)
	end

	_onTriggerUp = function()
		_OnTrigger("up", (active == 1) and 0 or 1, function(t)
			_pressed = false
			if _onUpCallback ~= nil then
				_onUpCallback()
			end
		end,
		"down", _onTriggerDown)
	end

	function self.IsPressed()
		return _pressed
	end

	function self.Connect(onDownCallback, onUpCallback)
		_onDownCallback = onDownCallback
		_onUpCallback = onUpCallback
	end

	_init()
	return self
end
