--
-- Elektronika dla Wszystkich
-- Szkoa Konstruktorw, zadanie 244 (EdW 06/2016)
-- ESP8266 IoT: WiFi Power Socket / Gniazdko sterowane przez WiFi
-- Copyright (c) 2016 by Marcin 'Malcom' Malich
-- Released under the MIT license
--

function BistableRelay(pinOn, pinOff)

	local self = {
		PinOn = pinOn,
		PinOff = pinOff,
	}

	local _driveTime = 75
	local _driveTimer = 0

	local _enabled = false

	local _init = function()
		print("BistableRelay: init " .. pinOn .. ", " .. pinOff)

		gpio.mode(pinOn, gpio.OUTPUT)
		gpio.write(pinOn, 0)
		gpio.mode(pinOff, gpio.OUTPUT)
		gpio.write(pinOff, 0)

		self.Reset()
	end

	local _driveRelay = function(on, off)
		print("BistableRelay: drive start: " .. on .. ", " .. off)

		-- cancel last operation if it's in progress to avoid situation
		-- when both pin are in high state in small period time
		tmr.stop(_driveTimer)
		gpio.write(pinOn, 0)
		gpio.write(pinOff, 0)

		gpio.write(pinOn, on)
		gpio.write(pinOff, off)

		tmr.alarm(_driveTimer, _driveTime, tmr.ALARM_SINGLE, function()
			gpio.write(pinOn, 0)
			gpio.write(pinOff, 0)
			print("BistableRelay: drive end")
		end)
	end

	function self.Set()
		print("BistableRelay: set")
		_driveRelay(1, 0)
		_enabled = true
	end

	function self.Reset()
		print("BistableRelay: reset")
		_driveRelay(0, 1)
		_enabled = false
	end

	function self.IsEnabled()
		return _enabled
	end

	function self.Enable(enable)
		print("BistableRelay: enable: "..tostring(enable))
		if enable == true then
			self.Set()
		else
			self.Reset()
		end
	end

	function self.Toggle()
		print("BistableRelay: toggle")
		if self.IsEnabled() == false then
			self.Set()
		else
			self.Reset()
		end
	end

	_init()
	return self
end
