--
-- Elektronika dla Wszystkich
-- Szkoa Konstruktorw, zadanie 244 (EdW 06/2016)
-- ESP8266 IoT: WiFi Power Socket / Gniazdko sterowane przez WiFi
-- Copyright (c) 2016 by Marcin 'Malcom' Malich
-- Released under the MIT license
--

Server = {}

local port = 80
local statusFunc
local actionFunc

local function ParseRequestInfo(request)
	-- GET /foo/bar(?key=val) HTTP/1.1
	local _, _, method, path, ver = string.find(request, '([A-Z]+) (.+) (HTTP/.+)')

	local pname = path
	local query = ''
	local pos = string.find(path, '?')
	if pos ~= nil then
		pname = string.sub(path, 1, pos - 1)
		query = string.sub(path, pos + 1)
	end

	local req = {}
	req.method = method
	req.path = path
	req.pathname = pname
	req.query = {}
	for k, v in string.gmatch(query, '(%w+)=(%w+)&*') do
		req.query[k] = v
	end
	req.version = ver

	return req
end

local function OnConnection(c)

	c:on('receive', function(sck, request)

		local req = ParseRequestInfo(request)

		print('Server: Incomming request: '..req.path);

		if (req.method ~= 'GET' or req.pathname ~= '/') then
			print('Invalid method or path')
			sck:send('HTTP/1.1 404 Not Found\r\n')
			return
		end

		local action = req.query['action']

		if (action == nil) then
			-- index page, do nothing

		elseif (action == 'enable' or action == 'disable') then
			print('Server: Action: '..action)
			actionFunc(action == 'enable')
		else
			-- invalid action, do nothing
			print('Server: Invalid action: '..action)
		end

		local status = statusFunc()
		if (status == true) then
			status = 'enabled'
		else
			status = 'disabled'
		end
		print('Server status: '..status)

		sck:send(
			'HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n'..
			'<!DOCTYPE html>'..
			'<html><head><title>ESP8266 IoT: WiFi Power Socket</title></head><body>'..
			'<h1>ESP8266 IoT: WiFi Power Socket</h1>'..
			'<p>Relay status: '..status..'<br/>'..
			'<a href="?action=enable">Enable</a> | <a href="?action=disable">Disable</a></p>'..
			'</body></html>'
		)

	end)

	c:on('sent', function(sck)
		sck:close()
	end)
end


function Server.Start(port, statusFunctor, actionFunctor)
	print("Server start at "..port)
	statusFunc = statusFunctor
	actionFunc = actionFunctor
	srv = net.createServer(net.TCP)
	srv:listen(port, OnConnection)
end
